﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Problems.QuadraticAssignment.QAPInstanceService;

namespace HeuristicLab.Problems.QuadraticAssignment {
  public class QAPService {
    private Dictionary<string, QAPInstanceDto> instanceCache;
    private Dictionary<string, QAPSolutionDto[]> solutionsCache;

    private void Initialize() {
      instanceCache = new Dictionary<string, QAPInstanceDto>();
      solutionsCache = new Dictionary<string, QAPSolutionDto[]>();
    }

    public bool IsCached(string instance) {
      return instanceCache != null && instanceCache.ContainsKey(instance);
    }

    public void ClearCache() {
      instanceCache.Clear();
      solutionsCache.Clear();
    }

    public void Configure(QuadraticAssignmentProblem problem, QAPClient client, string instance, Action<int> progressReporter) {
      if (string.IsNullOrEmpty(instance)) return;
      if (progressReporter != null) progressReporter(10);
      QAPInstanceDto data;
      if (instanceCache == null) Initialize();
      if (instanceCache.ContainsKey(instance)) data = instanceCache[instance];
      else {
        data = client.GetProblemInstanceData(instance);
        instanceCache.Add(instance, data);
      }
      if (progressReporter != null) progressReporter(50);
      DoubleMatrix weights = new DoubleMatrix(data.Weights.Length, data.Weights.Length);
      DoubleMatrix distances = new DoubleMatrix(data.Weights.Length, data.Weights.Length);
      try {
        for (int i = 0; i < data.Weights.Length; i++)
          for (int j = 0; j < data.Weights.Length; j++) {
            weights[i, j] = data.Weights[i][j];
            distances[i, j] = data.Distances[i][j];
          }
      } catch (IndexOutOfRangeException) {
        throw new InvalidOperationException("The problem data is malformed, the problem could not be loaded.");
      }
      if (progressReporter != null) progressReporter(65);
      problem.Name = data.Name;
      problem.Description = data.Description;
      problem.Maximization.Value = data.Maximization;
      problem.Weights = weights;
      problem.Distances = distances;

      problem.BestKnownQualityParameter.Value = null;
      problem.BestKnownSolution = null;
      problem.BestKnownSolutions = new ItemSet<Permutation>();
      QAPSolutionDto[] solutions;
      if (solutionsCache.ContainsKey(instance)) solutions = solutionsCache[instance];
      else {
        solutions = client.GetBestSolutionsData(instance);
        solutionsCache.Add(instance, solutions);
      }
      if (progressReporter != null) progressReporter(90);
      if (solutions.Any()) {
        problem.BestKnownQualityParameter.Value = new DoubleValue(solutions.First().Quality);
        if (solutions.First().Assignment != null && solutions.First().Assignment.Length > 0)
          problem.BestKnownSolution = new Permutation(PermutationTypes.Absolute, solutions.First().Assignment);
        foreach (var solution in solutions) {
          if (solution.Assignment != null && solution.Assignment.Length > 0)
            problem.BestKnownSolutions.Add(new Permutation(PermutationTypes.Absolute, solution.Assignment));
        }
      }
      if (progressReporter != null) progressReporter(100);
    }
  }
}
